import os
import numpy
from scipy.stats import linregress
import matplotlib
matplotlib.use('Agg')
from matplotlib import pyplot

# Constans
DIR = os.path.dirname(__file__)
DATADIR = os.path.join(DIR, 'fake_data')
OUTDIR = os.path.join(DIR, 'output')
if not os.path.isdir(OUTDIR):
    os.mkdir(OUTDIR)

# Get all data files in the data folder
all_files = os.listdir(DATADIR)

# Empty lists to hold data.
all_slopes = []
all_intercepts = []

# Loop through all files
for file_nr, filename in enumerate(all_files):
    
    # Check whether this is a data file
    name, ext = os.path.splitext(filename)
    # Skip this file if it's not a data file
    if ext != '.txt':
        continue
    if name[:7] != 'subject':
        continue

    # Open a data file and load data
    filepath = os.path.join(DATADIR, filename)
    raw = numpy.loadtxt(filepath, dtype=str, \
        delimiter='\t', unpack=True)
    
    # Count the number of columns in the file
    n_variables = len(raw)
    
    # Convert the data to a more convenient format
    data = {}
    for i in range(n_variables):
        varname = raw[i][0]
        values = raw[i][1:]
        try:
            data[varname] = values.astype(float)
        except:
            data[varname] = values
    
    # Make a Boolean vector for non-responses
    non_response = data['key'] == "None"
    # Make a Boolean vector for too fast responses
    too_quick = data['RT'] < 0.05
    # Combine the vectors using an OR operator
    exclude = non_response | too_quick
    
    # Select only the valid data
    rt = data['RT'][exclude==False]
    fp = data['foreperiod'][exclude==False]
    
    # Perform a linear regression
    model = linregress(fp, rt)
    slope = model[0]
    intercept = model[1]
    r = model[2]
    p = model[3]
    all_slopes.append(slope)
    all_intercepts.append(intercept)
    
    # Create a new figure
    fig, ax = pyplot.subplots(nrows=1, ncols=1)
    # Plot single participant data
    ax.plot(fp, rt, 'o', color='#FF69B4', \
        alpha=0.3)
    # Plot single participant model
    x = numpy.array([0, 7])
    y = intercept + x * slope
    ax.plot(x, y, '-', lw=3, color='#FF69B4')
    # Save the figure
    ax.set_xlabel("Foreperiod (sec)")
    ax.set_ylabel("Response time (sec)")
    figname = "%s.png" % (name)
    fig.savefig(os.path.join(OUTDIR, figname))
    pyplot.close(fig)

# Averaging
avg_slope = numpy.mean(all_slopes)
avg_intercept = numpy.mean(all_intercepts)
# Compute spread
sd_slope = numpy.std(all_slopes)
sd_intercept = numpy.std(all_intercepts)

# Compute the average model
x = numpy.array([0, 7])
y_avg = avg_intercept + x * avg_slope
y1 = avg_intercept - sd_intercept + \
    x * (avg_slope - sd_slope)
y2 = avg_intercept + sd_intercept + \
    x * (avg_slope + sd_slope)

# Generate a new figure
fig, ax = pyplot.subplots(nrows=1, ncols=1)
ax.plot(x, y_avg, '-', lw=3, color='#FF69B4')
ax.fill_between(x, y1, y2, color='#FF69B4', \
    alpha=0.3)
ax.set_xlabel("Foreperiod (sec)")
ax.set_ylabel("Response time (sec)")
fig.savefig("average_fit.png")

pyplot.close('all')
    
